/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: bezmesh.c,v $
 * Revision 1.1.2.2  1994/12/21  15:48:49  John_Dennis
 * 	new contributed code from SGI
 * 	[1994/12/21  15:42:03  John_Dennis]
 *
 * $EndLog$
 */
/*
 * (c) Copyright 1993, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */

/*  bezsurf.c
 *
 *  This program was adapted from bezsurf.c, which was published in
 *  "The OpenGL Programming Guide". 
 *
 *  All calls to auxlib were removed so this program is now X-specific. 
 *  This was done in order to demonstrate visual and colormap setup
 *  in a GLX program. For details on the steps involved in writing
 *  a GLX program refer to "OpenGL and X, Part 2: Using OpenGL with
 *  Xlib", by Mark J. Kilgard, in the Jan/Feb issue of "The X Journal".
 *
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include "win_utils.h"

GLfloat ctrlpoints[4][4][3] = {
    {{-1.5, -1.5, 4.0}, {-0.5, -1.5, 2.0}, 
	{0.5, -1.5, -1.0}, {1.5, -1.5, 2.0}}, 
    {{-1.5, -0.5, 1.0}, {-0.5, -0.5, 3.0}, 
	{0.5, -0.5, 0.0}, {1.5, -0.5, -1.0}}, 
    {{-1.5, 0.5, 4.0}, {-0.5, 0.5, 0.0}, 
	{0.5, 0.5, 3.0}, {1.5, 0.5, 4.0}}, 
    {{-1.5, 1.5, -2.0}, {-0.5, 1.5, -2.0}, 
	{0.5, 1.5, 0.0}, {1.5, 1.5, -1.0}}
};

void initlights(void)
{
    GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
    GLfloat position[] = { 0.0, 0.0, 2.0, 1.0 };
    GLfloat mat_diffuse[] = { 0.6, 0.6, 0.6, 1.0 };
    GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
    GLfloat mat_shininess[] = { 50.0 };
    
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
    
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
}

void mydisplay(void)
{
    glClear(GL_COLOR_BUFFER_BIT);
    glPushMatrix();
    glRotatef(85.0, 1.0, 1.0, 1.0);
    glEvalMesh2(GL_FILL, 0, 20, 0, 20);
    glPopMatrix();
    glFlush();
}

void myinit(void)
{
    glClearColor (0.0, 0.0, 0.0, 1.0);
    glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 4,
	    0, 1, 12, 4, &ctrlpoints[0][0][0]);
    glEnable(GL_MAP2_VERTEX_3);
    glEnable(GL_AUTO_NORMAL);
    glEnable(GL_NORMALIZE);
    glMapGrid2f(20, 0.0, 1.0, 20, 0.0, 1.0);
    initlights();	/* for lighted version only */
}

void myReshape(int w, int h)
{
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if (w <= h)
	glOrtho(-4.0, 4.0, -4.0*(GLfloat)h/(GLfloat)w, 
	    4.0*(GLfloat)h/(GLfloat)w, -4.0, 4.0);
    else
	glOrtho(-4.0*(GLfloat)w/(GLfloat)h, 
	    4.0*(GLfloat)w/(GLfloat)h, -4.0, 4.0, -4.0, 4.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void error_exit( char *error_message )
{
    fprintf( stderr, "%s\n", error_message);
    exit( 1);
}

void usage( char *name)
{
    fprintf( stderr, "\n%s%s%s%s%s%s\n",
        "usage: ", name, " [options]\n\n",
        "  Options:\n",
        "    -display  displayname  specify an X server connection\n",
        "    -geometry geometry     specify window geometry in pixels\n"
    );
    
    exit( 1);
}

#define TITLE "BEZIER MESH"
#define TRANSPARENT_PIXEL 0

int main(int argc, char** argv)
{

   char              *display = "";
   int               x=0, y=0;
   unsigned int      width=500, height=500;
   char              *geometry;
   int               i;

   Display           *dpy;
   XVisualInfo 	     vis_info;
   Colormap          cmap;
   Window            win;
   GLXContext        rd_context;
   int		     attr_mask;
   WinAttributes     attr;
   XEvent            event;   /* event record returned by X */
   int               done, redraw, overlay;
   KeySym 	     key_pressed;

   /* Get command line args */
   overlay = False;
   for (i=1; i < argc; i++) {
       if ( !strcmp(argv[i], "-geometry") ) {
           if (i++ > argc) usage( *argv);
           geometry = argv[i];
           XParseGeometry( geometry, &x, &y, &width, &height);
       } else if ( !strcmp(argv[i], "-display") ) {
           if (i++ > argc) usage( *argv);
           display = argv[i];
       } else if ( !strcmp(argv[i], "-overlay") ) {
           overlay = True;
       } else {
           usage( *argv);
       }
   }
   
   /* Step 1: Open Connection */
   dpy = open_glx_connection( display);
   if ( dpy == NULL ) error_exit( "Could not open connection");
   
   /* Step 2: Find visual and colormap */
   /* 
    * If overlay is True, then get transparent overlay visual otherwise
    * look for RGBA visual in the main planes 
    */
    
   if ( overlay ) {
       attr.transparent_type = WIN_TRANSPARENT;
       attr.level = 1;
       attr_mask = WIN_LEVEL_ATTR | WIN_TRANSPARENT_TYPE_ATTR;
   } else {
       attr_mask = WIN_COLOR_MODE_ATTR;
       attr.color_mode = WIN_RGBA;
   }

   if ( !get_visual_and_colormap( dpy, attr_mask, &attr, &vis_info, &cmap) )
       error_exit( "Could not find suitable visual and/or colormap");

   /* Step 3: Create window with visual and colormap */
   /*   
    * Note that this overlay example is atypical. Normally overlay windows are used
    * in conjunction with another window -- a toplevel window which resides in the
    * main image planes -- and are used for annotations, etc. For maximum portability 
    * an application should have a toplevel window and all other windows, including 
    * overlays, should be parented to it. To support this more typical (and more 
    * portable) usage of overlay planes, create_window takes the parent window as an 
    * argument.
    */
   win = create_window( dpy, RootWindow( dpy, vis_info.screen), TITLE, &vis_info,
                        cmap, x, y, width, height);

   /* Step 3a: Set the window background to be transparent pixel for the overlay window */
   if ( overlay )
      XSetWindowBackground( dpy, win, TRANSPARENT_PIXEL);

   /* Step 4: Select Input Events */
   XSelectInput( dpy, win, KeyPressMask | StructureNotifyMask | ExposureMask);

   /* Step 5: Create rendering context */
   rd_context = glXCreateContext( dpy, &vis_info, NULL, GL_TRUE);
   if ( rd_context == NULL ) error_exit( "Could not create rendering context");

   /* Step 6: Make the window current for the context */
   glXMakeCurrent( dpy, win, rd_context);

   /* Step 7: Map the window */
   XMapWindow( dpy, win);

   /* Step 8: Loop for events */
   myinit();
   done = False;
   while ( !done ) {
      do {
         XNextEvent( dpy, &event);
         switch ( event.type ) {
            case ConfigureNotify:
                myReshape( event.xconfigure.width, event.xconfigure.height);
                redraw = True;
                break;
            case Expose:
                redraw = True;
                break;
            case KeyPress:
                key_pressed = XLookupKeysym( (XKeyEvent*)&event, 0);
    		if ( key_pressed == XK_Escape ) done = True;
                break;
	    case ClientMessage:
                if ( decode_client_message( &event) == WIN_CLIENT_SHUTDOWN )
		    done = True;
	        break;
          }
      } while (XPending(dpy)); /* grab all events */

      if (redraw) {
          mydisplay();
	  /* If double-buffered window, then swap the buffers */
          if ( attr.ancillary_buffers & WIN_DOUBLEBUFFER )
	      glXSwapBuffers( dpy, win);
          redraw = False;
      }
   }

   /* Step 9: Terminate */
   XCloseDisplay( dpy);
   exit( 0);
}
