/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: get_visual.c,v $
 * Revision 1.1.2.2  1994/12/21  15:48:54  John_Dennis
 * 	new contributed code from SGI
 * 	[1994/12/21  15:42:18  John_Dennis]
 *
 * $EndLog$
 */
/*
 *  This program doesn't have any output. It simply chooses a visual that
 *  matches the specified command line arguments (visual attributes are 
 *  specified via command line arguments) and prints out the visual it 
 *  chose along with the attributes.
 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include "win_utils.h"

void error_exit( char *error_message )
{
    fprintf( stderr, "%s\n", error_message);
    exit( 1);
}

void usage( char *name)
{
    fprintf( stderr, "\n%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
        "usage: ", name, " [options]\n\n",
        "  Options:\n",
        "    -display  displayname  specify an X server connection\n",
        "    -rgba                  ask for rgba visual\n",
        "    -index                 ask for color index visual\n",
        "    -doublebuffer          ask for double buffered visual\n",
        "    -singlebuffer          ask for single buffered visual\n",
        "    -accum                 ask for accumulation buffer\n",
        "    -alpha                 ask for alpha buffer\n",
        "    -depth                 ask for depth buffer\n",
        "    -stencil               ask for stencil buffer\n",
        "    -aux nauxbuf           specify number of aux buffers\n",
        "    -level planes          specify planes (0=main,>0=overlay,<0=underlay\n",
        "    -transparent           ask for transparent overlay\n",
        "    -opaque                ask for opaque overlay\n"
    );
    
    exit( 1);
}

int main(int argc, char** argv)
{

   char              *display = "";
   int               i;

   Display           *dpy;
   XVisualInfo 	     vis_info;
   Colormap          cmap;
   WinAttributes     attr;
   int               mask;

   /* Get command line args
    * We allow all window attributes to be set via command line arguments
    */
   mask = attr.ancillary_buffers = 0;
   for (i=1; i < argc; i++) {
       if ( !strcmp(argv[i], "-display") ) {
           if (i++ > argc) usage( *argv);
           display = argv[i];
       } else if ( !strcmp(argv[i], "-rgba") ) {
           attr.color_mode = WIN_RGBA;
           mask |= WIN_COLOR_MODE_ATTR;
       } else if ( !strcmp(argv[i], "-index") ) {
           attr.color_mode = WIN_INDEX;
           mask |= WIN_COLOR_MODE_ATTR;
       } else if ( !strcmp(argv[i], "-mono") ) {
           attr.stereo = WIN_MONO;
           mask |= WIN_STEREO_ATTR;
       } else if ( !strcmp(argv[i], "-stereo") ) {
           attr.stereo = WIN_STEREO;
           mask |= WIN_STEREO_ATTR;
       } else if ( !strcmp(argv[i], "-doublebuffer") ) {
           attr.ancillary_buffers |= WIN_DOUBLEBUFFER;
	   mask |= WIN_ANCILLARY_BUFFERS_ATTR;
       } else if ( !strcmp(argv[i], "-singlebuffer") ) {
           attr.ancillary_buffers |= WIN_SINGLEBUFFER;
	   mask |= WIN_ANCILLARY_BUFFERS_ATTR;
       } else if ( !strcmp(argv[i], "-accum") ) {
           attr.ancillary_buffers |= WIN_ACCUMBUFFER;
	   mask |= WIN_ANCILLARY_BUFFERS_ATTR;
       } else if ( !strcmp(argv[i], "-alpha") ) {
           attr.ancillary_buffers |= WIN_ALPHABUFFER;
	   mask |= WIN_ANCILLARY_BUFFERS_ATTR;
       } else if ( !strcmp(argv[i], "-depth") ) {
           attr.ancillary_buffers |= WIN_DEPTHBUFFER;
	   mask |= WIN_ANCILLARY_BUFFERS_ATTR;
       } else if ( !strcmp(argv[i], "-stencil") ) {
           attr.ancillary_buffers |= WIN_STENCILBUFFER;
	   mask |= WIN_ANCILLARY_BUFFERS_ATTR;
       } else if ( !strcmp(argv[i], "-aux") ) {
	   if (i++ > argc) usage( *argv);
           attr.aux_buffers = atoi(argv[i]);
	   mask |= WIN_AUX_BUFFERS_ATTR;
       } else if ( !strcmp(argv[i], "-level") ) {
           if (i++ > argc) usage( *argv);
           attr.level = atoi( argv[i]);
	   mask |= WIN_LEVEL_ATTR;
       } else if ( !strcmp(argv[i], "-transparent") ) {
           attr.transparent_type = WIN_TRANSPARENT;
	   mask |= WIN_TRANSPARENT_TYPE_ATTR;
       } else if ( !strcmp(argv[i], "-opaque") ) {
           attr.transparent_type = WIN_OPAQUE;
	   mask |= WIN_TRANSPARENT_TYPE_ATTR;
       } else {
           usage( *argv);
       }
   }
   
   dpy = open_glx_connection( display);
   if ( dpy == NULL ) error_exit( "Could not open connection");
   
   if ( !get_visual_and_colormap( dpy, mask, &attr, &vis_info, &cmap) )
       error_exit( "Could not find suitable visual and/or colormap");

   fprintf( stderr, "Visual:\n");
   fprintf( stderr, "  visual i.d.: %d\n", vis_info.visualid);

   fprintf( stderr, "  class: ");
   switch ( vis_info.class ) {
        case StaticGray:
             fprintf( stderr, "static gray\n");
             break;
        case GrayScale:
             fprintf( stderr, "gray scale\n");
             break;
        case StaticColor:
             fprintf( stderr, "static color\n");
             break;
        case PseudoColor:
             fprintf( stderr, "pseudo color\n");
             break;
        case TrueColor:
             fprintf( stderr, "true color\n");
             break;
        case DirectColor:
             fprintf( stderr, "direct color\n");
             break;
   }
   fprintf( stderr, "  depth: %d\n", vis_info.depth);

   fprintf( stderr, "  color mode: ");
   if ( attr.color_mode == WIN_INDEX )
       fprintf( stderr, "index\n");
   else
       fprintf( stderr, "rgba\n");

   fprintf( stderr, "  stereo: ");
   if ( attr.stereo == WIN_STEREO )
       fprintf( stderr, "true\n");
   else
       fprintf( stderr, "false\n");

   fprintf( stderr, "  ancillary buffers: ");
   if ( attr.ancillary_buffers & WIN_DOUBLEBUFFER ) fprintf( stderr, "doublebuffer ");
   if ( attr.ancillary_buffers & WIN_SINGLEBUFFER ) fprintf( stderr, "singlebuffer ");
   if ( attr.ancillary_buffers & WIN_ACCUMBUFFER ) fprintf( stderr, "accum ");
   if ( attr.ancillary_buffers & WIN_ALPHABUFFER ) fprintf( stderr, "alpha ");
   if ( attr.ancillary_buffers & WIN_DEPTHBUFFER ) fprintf( stderr, "depth ");
   if ( attr.ancillary_buffers & WIN_STENCILBUFFER ) fprintf( stderr, "stencil ");
   fprintf( stderr, "\n");

   fprintf( stderr, "  aux buffers: %d\n", attr.aux_buffers);
   fprintf( stderr, "  level: %d\n", attr.level);
   fprintf( stderr, "  transparent: ");
   if ( attr.transparent_type == WIN_OPAQUE ) 
       fprintf( stderr, "false\n");
   else
       fprintf( stderr, "true\n");

   XCloseDisplay( dpy);
   exit( 0);
}
