/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: gl_ivp.c,v $
 * Revision 1.1.2.6  1995/07/20  20:11:52  Andy_Vesper
 * 	Make default mode `use default visual or first useable visual'
 * 	Also add calls to install colormaps and text to explain this
 * 	[1995/07/20  20:11:26  Andy_Vesper]
 *
 * Revision 1.1.2.5  1994/10/18  15:49:06  Andy_Vesper
 * 	Fix some picky things for the DEC C compiler
 * 	[1994/10/18  15:48:19  Andy_Vesper]
 * 
 * Revision 1.1.2.4  1994/07/21  15:28:32  Andy_Vesper
 * 	Add -visual ID and -allvisuals switches
 * 	[1994/07/21  15:26:35  Andy_Vesper]
 * 
 * Revision 1.1.2.3  1994/07/20  15:10:52  Andy_Vesper
 * 	Fix problem with visual selection on multihead
 * 	[1994/07/20  15:09:58  Andy_Vesper]
 * 
 * Revision 1.1.2.2  1994/05/24  16:09:39  Andy_Vesper
 * 	Initial version.
 * 	Initial version for v2.4
 * 	[1994/05/24  16:03:59  Andy_Vesper]
 * 
 * $EndLog$
 */
/*
 * (c) Copyright 1993, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */

/*  gl_ivp.c
 *
 *  This program was adapted from prim.c, which is part of the
 *  OpenGL distribution from SGI.
 *
 *  All calls to auxlib were removed so this program is now X-specific. 
 *  This was done in order to demonstrate visual and colormap setup
 *  in a GLX program. In particular, the default visual and colormap are
 *  used to avoid colormap thrashing on systems with only one
 *  concurrently installable colormap.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <GL/gl.h>
#include <GL/glx.h>

extern int sleep  (unsigned __seconds);

/* Manifest constants and a convenient macro */

#define TITLE	"OpenGL Installation Verification Program, visual 0x%x"
#define ICONTITLE  "OpenGL IVP 0x%x"

#define GAP	10
#define ROWS	3
#define COLS	4

#define PIXEL_CENTER(x) ((long)(x) + 0.5)

/* To support colors in both RGB and CI modes, this program uses the	     */
/* SETCOLOR macro with one of the COLOR_TAG enumeration constants	     */

#define SETCOLOR(_color)		\
{ if (ci) glIndexi (IndexMap[_color]);	\
  else glColor3fv (RGBMap[_color]);	\
}

enum COLOR_TAG
{
    BLACK,
    RED,
    GREEN,
    YELLOW,
    BLUE,
    MAGENTA,
    CYAN,
    WHITE
};

static GLfloat		RGBMap[8][3] =
{
    {
	0, 0, 0
    },
    {
	1, 0, 0
    },
    {
	0, 1, 0
    },
    {
	1, 1, 0
    },
    {
	0, 0, 1
    },
    {
	1, 0, 1
    },
    {
	0, 1, 1
    },
    {
	1, 1, 1
    }
};

static GLint		IndexMap[8];

/* A bitmap for glBitmap testing */

#define OPENGL_WIDTH 48
#define OPENGL_HEIGHT 13
static GLubyte OpenGL_bits[] = {
   0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 
   0x7f, 0xfb, 0xff, 0xff, 0xff, 0x01,
   0x7f, 0xfb, 0xff, 0xff, 0xff, 0x01, 
   0x00, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x3e, 0x8f, 0xb7, 0xf9, 0xfc, 0x01, 
   0x63, 0xdb, 0xb0, 0x8d, 0x0d, 0x00,
   0x63, 0xdb, 0xb7, 0x8d, 0x0d, 0x00, 
   0x63, 0xdb, 0xb6, 0x8d, 0x0d, 0x00,
   0x63, 0x8f, 0xf3, 0xcc, 0x0d, 0x00, 
   0x63, 0x00, 0x00, 0x0c, 0x4c, 0x0a,
   0x63, 0x00, 0x00, 0x0c, 0x4c, 0x0e, 
   0x63, 0x00, 0x00, 0x8c, 0xed, 0x0e,
   0x3e, 0x00, 0x00, 0xf8, 0x0c, 0x00, 
};

/* Masks for cycling through the colors in CI mode */

static GLuint		mask_all, mask_nored, mask_nogreen,
			mask_noblue;
				 
/* Shared variables */

static int		done;
static Display		*dpy;
static Window		win;
static GLenum		autoTest;
static int		sleepTime;
static char		*keyMacro;
static char		*keyMacroSpec;
static GLenum		ci;
static GLenum		doubleBuffer;
static GLint		windW;
static GLint		windH;
static GLenum		smoothShade;
static GLenum		FillMode;
static GLint		boxW;
static GLint		boxH;

static int		forceVisualID = 0;
static GLenum		allVisuals = GL_FALSE;
static Screen		* scr;
static Visual		* default_vi;
static Colormap		default_cmap;
static Window		rootWindow;
static Atom		wm_delete_window;

/* Routines */

static void error_exit(char *error_message )
{

    fprintf( stderr, "%s\n", error_message);
    exit( EXIT_FAILURE);

}

static void Init(void)
{

    smoothShade = GL_TRUE;
    FillMode = GL_TRUE;

}

static void Reshape(int width, int height)
{

    windW = (GLint)width;
    windH = (GLint)height;

}

static void RotateColorMask(void)
{
    static GLint rotation = 0;
    
    rotation = (rotation + 1) & 0x3;

    switch (rotation)
    {

    case 0:
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glIndexMask(mask_all );
	break;

    case 1:
	glColorMask(GL_FALSE, GL_TRUE, GL_TRUE, GL_TRUE);
	glIndexMask(mask_nored);
	break;

    case 2:
	glColorMask(GL_TRUE, GL_FALSE, GL_TRUE, GL_TRUE);
	glIndexMask(mask_nogreen);
	break;

    case 3:
	glColorMask(GL_TRUE, GL_TRUE, GL_FALSE, GL_TRUE);
	glIndexMask(mask_noblue);
	break;

    }

}

static GLenum Key(int key)
{

    switch (key)
    {

    case XK_Escape:	/* Key can be called with either an X11 keysym	     */
    case '\033':	/* or a ASCII character value			     */
    case '0':
	done = True;
	break;
	
    case XK_1:
	/* smoothShade is not used in CI mode */
	smoothShade = !smoothShade;
	break;

    case XK_2:
	FillMode = !FillMode;
	break;

    case XK_3:
	RotateColorMask();
	break;

    default:
	return GL_FALSE;
    }

    return GL_TRUE;

}

static void Viewport(GLint row, GLint column)
{
    GLint		x;
    GLint		y;

    boxW = (windW - (COLS + 1) * GAP) / COLS;
    boxH = (windH - (ROWS + 1) * GAP) / ROWS;

    x = GAP + column * (boxW + GAP);
    y = GAP + row * (boxH + GAP);

    glViewport(x, y, boxW, boxH);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-boxW/2, boxW/2, -boxH/2, boxH/2, 0.0, 1.0);
    glMatrixMode(GL_MODELVIEW);

    glEnable(GL_SCISSOR_TEST);
    glScissor(x, y, boxW, boxH);

}

static void Point(void)
{
    GLint		i;

    glBegin(GL_POINTS);
	SETCOLOR(WHITE);
	glVertex2i(0, 0);
	for (i = 1; i < 8; i++) {
	    GLint j = i * 2;
	    SETCOLOR(BLACK+i); /* OR: RED+i-1 */
	    glVertex2i(-j, -j);
	    glVertex2i(-j, 0);
	    glVertex2i(-j, j);
	    glVertex2i(0, j);
	    glVertex2i(j, j);
	    glVertex2i(j, 0);
	    glVertex2i(j, -j);
	    glVertex2i(0, -j);
	}
    glEnd();

}

static void Lines(void)
{
    GLint i;

    glPushMatrix();

    glTranslatef(-12, 0, 0);
    for (i = 1; i < 8; i++) {
	SETCOLOR(BLACK+i);
	glBegin(GL_LINES);
	    glVertex2i(-boxW/4, -boxH/4);
	    glVertex2i(boxW/4, boxH/4);
	glEnd();
	glTranslatef(4, 0, 0);
    }

    glPopMatrix();

}

static void LineStrip(void)
{

    glBegin(GL_LINE_STRIP);
	SETCOLOR(RED);
	glVertex2f(PIXEL_CENTER(-boxW/4), PIXEL_CENTER(-boxH/4));
	SETCOLOR(GREEN);
	glVertex2f(PIXEL_CENTER(-boxW/4), PIXEL_CENTER(boxH/4));
	SETCOLOR(BLUE);
	glVertex2f(PIXEL_CENTER(boxW/4), PIXEL_CENTER(boxH/4));
	SETCOLOR(WHITE);
	glVertex2f(PIXEL_CENTER(boxW/4), PIXEL_CENTER(-boxH/4));
    glEnd();

}

static void LineLoop(void)
{

    glBegin(GL_LINE_LOOP);
	SETCOLOR(RED);
	glVertex2f(PIXEL_CENTER(-boxW/4), PIXEL_CENTER(-boxH/4));
	SETCOLOR(GREEN);
	glVertex2f(PIXEL_CENTER(-boxW/4), PIXEL_CENTER(boxH/4));
	SETCOLOR(BLUE);
	glVertex2f(PIXEL_CENTER(boxW/4), PIXEL_CENTER(boxH/4));
	SETCOLOR(WHITE);
	glVertex2f(PIXEL_CENTER(boxW/4), PIXEL_CENTER(-boxH/4));
    glEnd();

    glEnable(GL_LOGIC_OP);
    glLogicOp(GL_XOR);

    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE, GL_ONE);

    SETCOLOR(MAGENTA);
    glBegin(GL_LINE_LOOP);
	glVertex2f(PIXEL_CENTER(-boxW/8), PIXEL_CENTER(-boxH/8));
	glVertex2f(PIXEL_CENTER(-boxW/8), PIXEL_CENTER(boxH/8));
    glEnd();
    glBegin(GL_LINE_LOOP);
	glVertex2f(PIXEL_CENTER(-boxW/8), PIXEL_CENTER(boxH/8+5));
	glVertex2f(PIXEL_CENTER(boxW/8), PIXEL_CENTER(boxH/8+5));
    glEnd();
    glDisable(GL_LOGIC_OP);
    glDisable(GL_BLEND);

    SETCOLOR(GREEN);
    glBegin(GL_POINTS);
	glVertex2i(0, 0);
    glEnd();

}

static void Bitmap(void)
{

    glBegin(GL_LINES);
	SETCOLOR(GREEN);
	glVertex2i(-boxW/2, 0);
	glVertex2i(boxW/2, 0);
	glVertex2i(0, -boxH/2);
	glVertex2i(0, boxH/2);
	SETCOLOR(RED);
	glVertex2i(0, -3);
	glVertex2i(0, -3+OPENGL_HEIGHT);
	SETCOLOR(BLUE);
	glVertex2i(0, -3);
	glVertex2i(OPENGL_WIDTH, -3);
    glEnd();

    SETCOLOR(GREEN);

    glPixelStorei(GL_UNPACK_LSB_FIRST, GL_TRUE);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

    glRasterPos2i(0, 0);
    glBitmap(OPENGL_WIDTH, OPENGL_HEIGHT, 0, 3, 0.0, 0.0, OpenGL_bits);
}

static void Triangles(void)
{

    glBegin(GL_TRIANGLES);
	SETCOLOR(GREEN);
	glVertex2i(-boxW/4, -boxH/4);
	SETCOLOR(RED);
	glVertex2i(-boxW/8, -boxH/16);
	SETCOLOR(BLUE);
	glVertex2i(boxW/8, -boxH/16);

	SETCOLOR(GREEN);
	glVertex2i(-boxW/4, boxH/4);
	SETCOLOR(RED);
	glVertex2i(-boxW/8, boxH/16);
	SETCOLOR(BLUE);
	glVertex2i(boxW/8, boxH/16);
    glEnd();

}

static void TriangleStrip(void)
{

    glBegin(GL_TRIANGLE_STRIP);
	SETCOLOR(GREEN);
	glVertex2i(-boxW/4, -boxH/4);
	SETCOLOR(RED);
	glVertex2i(-boxW/4, boxH/4);
	SETCOLOR(BLUE);
	glVertex2i(0, -boxH/4);
	SETCOLOR(WHITE);
	glVertex2i(0, boxH/4);
	SETCOLOR(CYAN);
	glVertex2i(boxW/4, -boxH/4);
	SETCOLOR(YELLOW);
	glVertex2i(boxW/4, boxH/4);
    glEnd();

    glBegin(GL_TRIANGLE_STRIP);
	glVertex2i(0, 0);
	glVertex2i(-100, 100);
    glEnd();
}

static void TriangleFan(void)
{
    GLint vx[8][2];
    GLint x0, y0, x1, y1, x2, y2, x3, y3;
    GLint i;

    y0 = -boxH/4;
    y1 = y0 + boxH/2/3;
    y2 = y1 + boxH/2/3;
    y3 = boxH/4;
    x0 = -boxW/4;
    x1 = x0 + boxW/2/3;
    x2 = x1 + boxW/2/3;
    x3 = boxW/4;

    vx[0][0] = x0; vx[0][1] = y1;
    vx[1][0] = x0; vx[1][1] = y2;
    vx[2][0] = x1; vx[2][1] = y3;
    vx[3][0] = x2; vx[3][1] = y3;
    vx[4][0] = x3; vx[4][1] = y2;
    vx[5][0] = x3; vx[5][1] = y1;
    vx[6][0] = x2; vx[6][1] = y0;
    vx[7][0] = x1; vx[7][1] = y0;

    glBegin(GL_TRIANGLE_FAN);
	SETCOLOR(WHITE);
	glVertex2i(0, 0);
	for (i = 0; i < 8; i++) {
	    SETCOLOR(WHITE-i);
	    glVertex2iv(vx[i]);
	}
    glEnd();

}

static void Rect(void)
{

    SETCOLOR(GREEN);
    glRecti(-boxW/4, -boxH/4, boxW/4, boxH/4);
}

static void Polygon(void)
{
    GLint vx[8][2];
    GLint x0, y0, x1, y1, x2, y2, x3, y3;
    GLint i;

    y0 = -boxH/4;
    y1 = y0 + boxH/2/3;
    y2 = y1 + boxH/2/3;
    y3 = boxH/4;
    x0 = -boxW/4;
    x1 = x0 + boxW/2/3;
    x2 = x1 + boxW/2/3;
    x3 = boxW/4;

    vx[0][0] = x0; vx[0][1] = y1;
    vx[1][0] = x0; vx[1][1] = y2;
    vx[2][0] = x1; vx[2][1] = y3;
    vx[3][0] = x2; vx[3][1] = y3;
    vx[4][0] = x3; vx[4][1] = y2;
    vx[5][0] = x3; vx[5][1] = y1;
    vx[6][0] = x2; vx[6][1] = y0;
    vx[7][0] = x1; vx[7][1] = y0;

    glBegin(GL_POLYGON);
	for (i = 0; i < 8; i++) {
	    SETCOLOR(WHITE-i);
	    glVertex2iv(vx[i]);
	}
    glEnd();

}

static void Quads(void)
{

    glBegin(GL_QUADS);
	SETCOLOR(GREEN);
	glVertex2i(-boxW/4, -boxH/4);
	SETCOLOR(RED);
	glVertex2i(-boxW/8, -boxH/16);
	SETCOLOR(BLUE);
	glVertex2i(boxW/8, -boxH/16);
	SETCOLOR(WHITE);
	glVertex2i(boxW/4, -boxH/4);

	SETCOLOR(GREEN);
	glVertex2i(-boxW/4, boxH/4);
	SETCOLOR(RED);
	glVertex2i(-boxW/8, boxH/16);
	SETCOLOR(BLUE);
	glVertex2i(boxW/8, boxH/16);
	SETCOLOR(WHITE);
	glVertex2i(boxW/4, boxH/4);
    glEnd();

}

static void QuadStrip(void)
{

    glBegin(GL_QUAD_STRIP);
	SETCOLOR(GREEN);
	glVertex2i(-boxW/4, -boxH/4);
	SETCOLOR(RED);
	glVertex2i(-boxW/4, boxH/4);
	SETCOLOR(BLUE);
	glVertex2i(0, -boxH/4);
	SETCOLOR(WHITE);
	glVertex2i(0, boxH/4);
	SETCOLOR(CYAN);
	glVertex2i(boxW/4, -boxH/4);
	SETCOLOR(YELLOW);
	glVertex2i(boxW/4, boxH/4);
    glEnd();

}

static void Draw(void)
{

    glViewport(0, 0, windW, windH);
    glDisable(GL_SCISSOR_TEST);

    glPushAttrib(GL_COLOR_BUFFER_BIT);

    glColorMask(1, 1, 1, 1);
    glIndexMask(~0);

    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClearIndex(IndexMap[0]);
    glClear(GL_COLOR_BUFFER_BIT);

    glPopAttrib();

    if (smoothShade && !ci) {
	glShadeModel(GL_SMOOTH);
    } else {
	glShadeModel(GL_FLAT);
    }

    if (FillMode) {
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    } else {
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    }

    Viewport(0, 0); Point();
    Viewport(0, 1); Lines();
    Viewport(0, 2); LineStrip();
    Viewport(0, 3); LineLoop();

    Viewport(1, 0); Bitmap();
    Viewport(1, 1); TriangleFan();
    Viewport(1, 2); Triangles();
    Viewport(1, 3); TriangleStrip();

    Viewport(2, 0); Rect();
    Viewport(2, 1); Polygon();
    Viewport(2, 2); Quads();
    Viewport(2, 3); QuadStrip();

    glFlush();

}

static GLenum Args(int argc, char **argv)
{
    GLint		i;

    autoTest = GL_TRUE;
    sleepTime = 1;
    keyMacroSpec = NULL;

    for (i = 1; i < argc; i++)
    {
	if (strcmp(argv[i], "-noauto") == 0)
	{
	    autoTest = GL_FALSE;
	}
	else if (strcmp(argv[i], "-auto") == 0)
	{
	    autoTest = GL_TRUE;
	}
	else if (strcmp(argv[i], "-sleepTime") == 0
		|| strcmp(argv[i], "-sleeptime") == 0)
	{
	    i++;
	    if (i >= argc)
	    {
		printf("no time specified\n");
		return GL_FALSE;
	    }
	    if (sscanf(argv[i], "%d", &sleepTime) != 1)
	    {
		printf("%s (Bad sleepTime).\n", argv[i]);
		return GL_FALSE;
	    }
	}
	else if (strcmp(argv[i], "-macro") == 0)
	{
	    i++;
	    if (i >= argc)
	    {
		printf("no macro specified\n");
		return GL_FALSE;
	    }
	    keyMacroSpec = argv[i];
	}
	else if (strcmp(argv[i], "-visual") == 0)
	{
	    i++;
	    if (i >= argc)
	    {
		printf("no visual specified\n");
		return GL_FALSE;
	    }
	    forceVisualID = strtol (argv[i], NULL, 0);
	}
	else if (strcmp(argv[i], "-allvisuals") == 0)
	{
	    allVisuals = GL_TRUE;
	}
	else
	{
	    printf("%s (Bad option).\n", argv[i]);
	    printf("Options are:\n");
	    printf("   -noauto           run manually\n");
	    printf("   -macro <string>   use <string> as the auto commands\n");
	    printf("   -sleepTime <time> use <time> (seconds) as a delay\n");
	    printf("   -visual <number>  use visual id given\n");
	    printf("   -allvisuals       try all visuals\n");
	    return GL_FALSE;
	}

    }

    if (forceVisualID != 0 && allVisuals == GL_TRUE)
    {
	printf ("-visual %d overrules -allvisuals\n", forceVisualID);
	allVisuals = GL_FALSE;
    }

    return GL_TRUE;

}

static void DoAutoTest( void )
{
    char		*p;

    Draw();
    if (doubleBuffer)
	glXSwapBuffers( dpy, win);

    glFinish();
    sleep(sleepTime);

    for (p = keyMacro; *p; p++)
    {
	Key(*p);
	Draw();
	if (doubleBuffer) glXSwapBuffers( dpy, win);
	glFinish();

	sleep(sleepTime);
    }

}


static void test_visual (XVisualInfo * vi_info)
{
    int			i;
    Colormap		cmap;
    GLXContext		rd_context;
    XEvent		event;   /* event record returned by X */
    int			redraw; 
    KeySym		key_pressed;
    int			gl_supported;
    XSizeHints		win_hint; /* window manager hints */
    XSetWindowAttributes win_attr;
    Atom		wm_delete_window;
    unsigned long	base_pixel, masks[3];
    GLint		depth_size;
    GLint		red_size, green_size, blue_size, alpha_size;
    char		title[100];
    char		icontitle[100];

    printf ("\n");

    /* Make sure the OpenGL extension supports this visual */

    gl_supported = GL_FALSE;
    (void) glXGetConfig (dpy, vi_info, GLX_USE_GL, &gl_supported);
    if (gl_supported == GL_FALSE)
    {
	printf ("OpenGL does not support visual 0x%x\n", vi_info->visualid);
	return;
    }

    /* Did we get a single or double buffered visual? */
    
    (void) glXGetConfig (dpy, vi_info, GLX_DOUBLEBUFFER, &gl_supported);
    if (gl_supported == GL_TRUE) doubleBuffer = GL_TRUE;
    else doubleBuffer = GL_FALSE;

    /* Check what visual class we found; we won't try to support everthing   */
    /* because we can't do index masking without being able to control the   */
    /* pixel allocation.						     */

    switch (vi_info->class)
    {

    case StaticGray:
	printf ("Cannot handle StaticGray visual 0x%x\n", vi_info->visualid);
	return;

    case StaticColor:
	printf ("Cannot handle StaticColor visual 0x%x\n", vi_info->visualid);
	return;

    case PseudoColor:
    case GrayScale:
	(void) glXGetConfig (dpy, vi_info, GLX_BUFFER_SIZE, &depth_size);
	
	if (vi_info->visual == default_vi) printf ("Default ");
	printf ("Visual id 0x%x; class %s, %s-buffered\n",
	    vi_info->visualid,
	    (vi_info->class == PseudoColor ? "PseudoColor" : "GrayScale"),
	    (doubleBuffer ? "double" : "single")
	    );

	printf ("Color Index mode; buffer size in bits: %d\n", depth_size);
	ci = GL_TRUE;

	if (vi_info->visual == default_vi)
	{
	    cmap = default_cmap;
	}
	else
	{
	    cmap = XCreateColormap (dpy, rootWindow, vi_info->visual,
		    AllocNone);
	    if (cmap == (Colormap) 0)
	    {
		printf ("Could not create colormap for visual 0x%x\n",
			vi_info->visualid);
		return;	
	    }
	}

	/* We need to allocate the right colors from the colormap,	     */
	/* if we can.							     */
	/* We also want the colors in a certain arrangement so we can	     */
	/* use the index mask and get the same colors consistantly.	     */
	/* Therefore, we must use XAllocColorCells			     */

	if (XAllocColorCells (dpy, cmap, True, masks, 3 /* planes */,
		&base_pixel, 1) == 0)
	{
	    printf ("Could not allocate 8 colors for visual 0x%x",
		    vi_info->visualid);
	}

	/* The pixels we allocated use all combinations of the three masks,   */
	/* each or'ed with the base pixel				     */

	IndexMap[0] = base_pixel;
	IndexMap[1] = base_pixel			| masks[0];
	IndexMap[2] = base_pixel	    | masks[1];
	IndexMap[3] = base_pixel	    | masks[1]	| masks[0];
	IndexMap[4] = base_pixel | masks[2];
	IndexMap[5] = base_pixel | masks[2]		| masks[0];
	IndexMap[6] = base_pixel | masks[2] | masks[1];
	IndexMap[7] = base_pixel | masks[2] | masks[1]	| masks[0];

	/* Set up the masks for color cycling.				     */

	mask_all = ~0;
	mask_nored = ~masks[0];
	mask_nogreen = ~masks[1];
	mask_noblue = ~masks[2];

	/* Store the appropriate color for each pixel (index).		     */

	for (i = 0; i < 8; i++)
	{
	    XColor	c;

	    c.pixel = IndexMap[i];
	    c.red = (unsigned short) 0xffff * RGBMap[i][0];
	    c.green = (unsigned short) 0xffff * RGBMap[i][1];
	    c.blue = (unsigned short) 0xffff * RGBMap[i][2];
	    c.flags = DoRed | DoGreen | DoBlue;
	    XStoreColor (dpy, cmap, &c);
	}
	break;

    case TrueColor:
	if (vi_info->visual == default_vi)
	{
	    cmap = default_cmap;
	}
	else
	{
	    cmap = XCreateColormap (dpy, rootWindow, vi_info->visual,
		    AllocNone);
	    if (cmap == (Colormap) 0)
	    {
		printf ("Could not create colormap for visual 0x%x\n",
			vi_info->visualid);
		return;	
	    }
	}
	(void) glXGetConfig (dpy, vi_info, GLX_RED_SIZE, &red_size);
	(void) glXGetConfig (dpy, vi_info, GLX_GREEN_SIZE, &green_size);
	(void) glXGetConfig (dpy, vi_info, GLX_BLUE_SIZE, &blue_size);
	(void) glXGetConfig (dpy, vi_info, GLX_ALPHA_SIZE, &alpha_size);

	if (vi_info->visual == default_vi) printf ("Default ");
	printf ("Visual id 0x%x; class TrueColor, %s-buffered\n",
	    vi_info->visualid,
	    (doubleBuffer ? "double" : "single")
	    );
	printf (
	"RGB mode; buffer size in bits: red %d, green %d, blue %d, alpha %d\n",
	    red_size, green_size, blue_size, alpha_size);
	ci = GL_FALSE;
	break;

    case DirectColor:
	printf ("Cannot handle DirectColor visual 0x%x\n",
		vi_info->visualid);
	return;
    }
    
    if (keyMacroSpec != NULL)
    {
	keyMacro = keyMacroSpec;
    }
    else
    {
	/* setup the default macro based on CI or RGB; we don't do	     */
	/* smooth shading in CI mode because it won't look right.	     */
	if (ci)	keyMacro = "3333233330";
	else keyMacro = "113333233330";
    }

    /* We now have a visual and a colormap; create the window		     */

    windW = 600;
    windH = 300;

    win_hint.x = 0;
    win_hint.y = 0;
    win_hint.width = windW; win_hint.height = windH;
    win_hint.flags = PPosition | PSize;

    /* Create the window using the specified visual and colormap. */

    win_attr.colormap = cmap;

    /* We have to specify border pixel if visual is not the default.	     */
    /* This will never be the case for this program, but it is necessary     */
    /* for programs that might select a visual not the default.		     */

    win_attr.border_pixel = 0;

    win = XCreateWindow( dpy, RootWindowOfScreen(scr),
        win_hint.x, win_hint.y, windW, windH,
        0 /*border width*/, vi_info->depth, InputOutput, vi_info->visual,
        CWColormap | CWBorderPixel, &win_attr);

    /* Now set the window title and size hints */

    sprintf (title, TITLE, vi_info->visualid);
    sprintf (icontitle, ICONTITLE, vi_info->visualid);
    XSetStandardProperties( dpy, win, title, icontitle, None, NULL,
	    0, &win_hint);

    /* Allow clean termination from window manager */

    if ( wm_delete_window != None )
	    XSetWMProtocols( dpy, win, &wm_delete_window, 1);

    /* Select Input Events */

    XSelectInput( dpy, win, KeyPressMask | StructureNotifyMask | ExposureMask);

    /* Create rendering context */

    rd_context = glXCreateContext( dpy, vi_info, NULL, GL_TRUE);
    if ( rd_context == NULL )
    {
	printf ("Could not create rendering context for visual 0x%x\n",
		vi_info->visualid);
	XDestroyWindow (dpy, win);
	if (cmap != default_cmap)
	{
	    XFreeColormap (dpy, cmap);
	}
    }

    /* Make the window current for the context */

    glXMakeCurrent( dpy, win, rd_context);

    /* Map the window */

    XMapWindow( dpy, win);
    XInstallColormap (dpy, cmap);

    /* Loop for events */

    Init();
    done = False;

    while ( !done )
    {

	do
	{

	    XNextEvent( dpy, &event);

	    switch ( event.type )
	    {

            case ConfigureNotify:
                Reshape( event.xconfigure.width, event.xconfigure.height);
                redraw = True;
                break;

            case Expose:
                redraw = True;
                break;

            case KeyPress:
                key_pressed = XLookupKeysym( (XKeyEvent*)&event, 0);
		Key (key_pressed);
		redraw = True;
                break;

	    case ClientMessage:
                if (event.xclient.data.l[0] == wm_delete_window)
		    done = True;
	        break;
	    }

	} while ( XPending(dpy) ); /* Process all events before drawing */

	if (!done && redraw)
	{
	    if (autoTest)
	    {
		DoAutoTest ();
		redraw = False;
	    }
	    else
	    {
		Draw();
		if (doubleBuffer) glXSwapBuffers( dpy, win);
		redraw = False;
	    }
	}   /* end if (!done && redraw) */

    }	    /* end while (!done) */

    /* clean up */

    glXMakeCurrent (dpy, (Window) 0, NULL);
    glXDestroyContext (dpy, rd_context);
    XDestroyWindow (dpy, win);
    if (cmap != default_cmap)
    {
	XUninstallColormap (dpy, cmap);
	XFreeColormap (dpy, cmap);
    }

}

int main(int argc, char** argv)
{
    int			i;
    XVisualInfo 	vis_info_template;
    XVisualInfo 	* vi_info;
    int			error, eventnum;
    int			num_visuals;
    int			gl_supported;

    printf ("\nThis is the OpenGL Installation Verification Program\n");
    printf ("\nThis IVP will install colormaps for its test windows.\n");
    printf ("Other windows may go technicolor; please wait till the\n");
    printf ("IVP is complete and then assign focus to (click on) any\n");
    printf ("window to cause its colormap to be installed.\n");
    
    /* Check the arguments from the user */

    if (Args (argc, argv) == GL_FALSE) exit (EXIT_FAILURE);

    /* Open the display */
    
    dpy = XOpenDisplay (NULL);
    if ( dpy == NULL ) error_exit( "Could not open X11 display");
    if (glXQueryExtension (dpy, &error, &eventnum) == False)
    {
	error_exit ("No OpenGL extension on display");
    }

    wm_delete_window = XInternAtom( dpy, "WM_DELETE_WINDOW", False);

    /* Get information on the default screen, visual and colormap */

    scr = DefaultScreenOfDisplay (dpy);
    default_vi = DefaultVisualOfScreen (scr);
    default_cmap = DefaultColormapOfScreen (scr);
    rootWindow = RootWindowOfScreen (scr);
    vis_info_template.screen = DefaultScreen(dpy);

    if (allVisuals)
    {
	vi_info = XGetVisualInfo (dpy, VisualScreenMask,
		&vis_info_template, &num_visuals);
	if (num_visuals == 0)
	{
	    error_exit ("Could not get visual information for this display");
	}
	for (i = 0; i < num_visuals; i++)
	{
	    test_visual (vi_info);
	    vi_info++;
	}
    }
    else if (forceVisualID != 0)
    {
	vis_info_template.visualid = forceVisualID;
	vi_info = XGetVisualInfo (dpy, VisualIDMask | VisualScreenMask,
		&vis_info_template, &num_visuals);
	if (vi_info == NULL || num_visuals == 0)
	{
	    printf ("Could not get default visual information for visual 0x%x\n",
		    vis_info_template.visualid);
	    exit (EXIT_FAILURE);
	}
	test_visual (vi_info);
    }
    else
    {
	/* Default action -- check to see if the default visual is	     */
	/* useable by OpenGL. If so, test that. If the default visual is     */
	/* NOT useable, search the list to find one that is.		     */

	vis_info_template.visualid = XVisualIDFromVisual(default_vi);
	vi_info = XGetVisualInfo (dpy, VisualIDMask | VisualScreenMask,
		&vis_info_template, &num_visuals);
	if (vi_info == NULL || num_visuals == 0)
	{
	    printf (
	"Could not get default visual information for default visual 0x%x\n",
		    vis_info_template.visualid);
	    exit (EXIT_FAILURE);
	}

	gl_supported = GL_FALSE;
	(void) glXGetConfig (dpy, vi_info, GLX_USE_GL, &gl_supported);
	if (gl_supported)
	{
	    test_visual (vi_info);
	}
	else
	{
	    vi_info = XGetVisualInfo (dpy, VisualScreenMask,
		    &vis_info_template, &num_visuals);
	    if (num_visuals == 0)
	    {
		error_exit ("Could not get visual information for this display");
	    }

	    gl_supported = GL_FALSE;
	    for (i = 0; i < num_visuals; i++)
	    {
		(void) glXGetConfig (dpy, vi_info, GLX_USE_GL, &gl_supported);
		if (gl_supported)
		{
		    test_visual (vi_info);
		    break;
		}
		vi_info++;
	    }
	    if (gl_supported == GL_FALSE)
	    {
		/* No visuals are supported! Something is wrong. */
		error_exit ("Could not find any visual supported by OpenGL\n");
	    }
	}

    }

    /* Attempt to put the `right' colormap in place */
    XInstallColormap (dpy, default_cmap);

    printf ("\nThe OpenGL IVP is now complete. If your colors are wrong,\n");
    printf ("please assign focus to (click on) any window to cause its\n");
    printf ("colormap to be installed.\n");

    /* Terminate; let the X11 server clean up after us. */

    XCloseDisplay( dpy);

    exit( EXIT_SUCCESS);

}
