/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: rand48.c,v $
 * Revision 1.1.2.3  1996/02/16  15:16:36  Andy_Vesper
 * 	Glut 3.0 for R6
 * 	[1996/02/16  14:02:55  Andy_Vesper]
 *
 * Revision 1.1.2.3  1996/02/12  20:24:16  Andy_Vesper
 * 	Glut 3.0
 * 	[1996/02/12  20:05:07  Andy_Vesper]
 * 
 * Revision 1.1.2.2  1995/10/24  21:36:13  Andy_Vesper
 * 	rand48 function for OpenVMS
 * 	[1995/10/24  20:20:37  Andy_Vesper]
 * 
 * $EndLog$
 */
/*
 * Copyright (c) 1993 Martin Birgmeier
 * All rights reserved.
 *
 * You may redistribute unmodified or modified versions of this source
 * code provided that the above copyright notice and this and the
 * following conditions are retained.
 *
 * This software is provided ``as is'', and comes with no warranties
 * of any kind. I shall in no event be liable for anything that happens
 * to anyone/anything when using this software.
 */

#include <math.h>
#include <stdlib.h>

#define	RAND48_SEED_0	(0x330e)
#define	RAND48_SEED_1	(0xabcd)
#define	RAND48_SEED_2	(0x1234)
#define	RAND48_MULT_0	(0xe66d)
#define	RAND48_MULT_1	(0xdeec)
#define	RAND48_MULT_2	(0x0005)
#define	RAND48_ADD	(0x000b)

unsigned short _rand48_seed[3] = {
	RAND48_SEED_0,
	RAND48_SEED_1,
	RAND48_SEED_2
};
unsigned short _rand48_mult[3] = {
	RAND48_MULT_0,
	RAND48_MULT_1,
	RAND48_MULT_2
};
unsigned short _rand48_add = RAND48_ADD;

extern double drand48 (void);
extern void srand48 (long seed);

int random (void)
{
    unsigned long accu;
    unsigned short temp[2];

    accu = (unsigned long) _rand48_mult[0] * (unsigned long) _rand48_seed[0] +
     (unsigned long) _rand48_add;
    temp[0] = (unsigned short) accu;	/* lower 16 bits */
    accu >>= sizeof(unsigned short) * 8;
    accu += (unsigned long) _rand48_mult[0] * (unsigned long) _rand48_seed[1] +
     (unsigned long) _rand48_mult[1] * (unsigned long) _rand48_seed[0];
    temp[1] = (unsigned short) accu;	/* middle 16 bits */
    accu >>= sizeof(unsigned short) * 8;
    accu += _rand48_mult[0] * _rand48_seed[2] + _rand48_mult[1] * _rand48_seed[1] + _rand48_mult[2] * _rand48_seed[0];
    _rand48_seed[0] = temp[0];
    _rand48_seed[1] = temp[1];
    _rand48_seed[2] = (unsigned short) accu;

    return ( (_rand48_seed[0] << 16) + _rand48_seed[1]) & 0x7fffffff;
}

void srandom (unsigned int seed)
{
    srand48 ( (long) seed);
}


double
drand48(void)
{
    unsigned long accu;
    unsigned short temp[2];

    accu = (unsigned long) _rand48_mult[0] * (unsigned long) _rand48_seed[0] +
     (unsigned long) _rand48_add;
    temp[0] = (unsigned short) accu;	/* lower 16 bits */
    accu >>= sizeof(unsigned short) * 8;
    accu += (unsigned long) _rand48_mult[0] * (unsigned long) _rand48_seed[1] +
     (unsigned long) _rand48_mult[1] * (unsigned long) _rand48_seed[0];
    temp[1] = (unsigned short) accu;	/* middle 16 bits */
    accu >>= sizeof(unsigned short) * 8;
    accu += _rand48_mult[0] * _rand48_seed[2] + _rand48_mult[1] * _rand48_seed[1] + _rand48_mult[2] * _rand48_seed[0];
    _rand48_seed[0] = temp[0];
    _rand48_seed[1] = temp[1];
    _rand48_seed[2] = (unsigned short) accu;

    return ldexp((double) _rand48_seed[0], -48) +
	   ldexp((double) _rand48_seed[1], -32) +
	   ldexp((double) _rand48_seed[2], -16);
}


void
srand48(long seed)
{
    _rand48_seed[0] = RAND48_SEED_0;
    _rand48_seed[1] = (unsigned short) seed;
    _rand48_seed[2] = (unsigned short) (seed >> 16);
    _rand48_mult[0] = RAND48_MULT_0;
    _rand48_mult[1] = RAND48_MULT_1;
    _rand48_mult[2] = RAND48_MULT_2;
    _rand48_add = RAND48_ADD;
}
