/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: speed.c,v $
 * Revision 1.1.2.7  1994/01/21  17:22:50  John_Dennis
 * 	clean up to match SGI 1.0.12 release
 * 	[1994/01/21  17:19:26  John_Dennis]
 *
 * Revision 1.1.2.6  1994/01/11  18:16:40  Andy_Vesper
 * 	Fix use of sys$gettim and lib$sub_times
 * 	[1994/01/11  18:10:16  Andy_Vesper]
 * 
 * Revision 1.1.2.5  1993/12/08  21:40:51  Andy_Vesper
 * 	Add extra text to output so that you can understand
 * 	the results. Also define sys$gettim and lib$sub_times
 * 	to avoid VMS compiler messages.
 * 	[1993/12/08  21:38:16  Andy_Vesper]
 * 
 * Revision 1.1.2.4  1993/12/03  20:49:35  Andy_Vesper
 * 	Add define of __unix if DEC and __unix__ are defined.
 * 	This will give us timing on OSF/1.
 * 	Also do timing for VMS.
 * 	Finally, fix a number of missing break bugs and disable
 * 	lots of things when needed.
 * 	[1993/12/03  20:48:38  Andy_Vesper]
 * 
 * Revision 1.1.2.3  1993/11/10  20:53:30  Andy_Vesper
 * 	Changes needed for compiling on OpenVMS
 * 	[1993/11/10  19:31:41  Andy_Vesper]
 * 
 * Revision 1.1.2.2  1993/10/07  20:14:48  John_Dennis
 * 	new files in SGI OpenGL 1.0.11 release
 * 	[1993/10/07  18:54:06  John_Dennis]
 * 
 * $EndLog$
 */
#include <stdio.h>
#include <string.h>
#ifdef DEC
# ifdef vms
extern int sys$gettim (long *);
extern int lib$sub_times (long *, long *, long *);
# else /* not VMS */
# include <unistd.h>
# define __unix 1
# endif /* not VMS */
#else /* not DEC */
#include <unistd.h>
#endif /* not DEC */
#include <stdlib.h>
#include "tk.h"

#ifdef __unix
#include <sys/times.h>
#include <sys/param.h>
#endif
#ifdef DEC
#ifndef HZ
# ifdef CLK_TCK
#   define HZ CLK_TCK
# else
#   define HZ 60
# endif
#endif
#endif


#define GAP 10
#define ROWS 1
#define COLS 4


GLenum rgb, doubleBuffer, directRender, windType;
GLint windW, windH;

GLint boxW, boxH;

GLenum antialiasing = GL_FALSE;
GLenum depthTesting = GL_FALSE;
GLenum fogging = GL_FALSE, niceFogging = GL_FALSE;
GLenum lighting = GL_FALSE;
GLenum shading = GL_FALSE;
GLenum texturing = GL_FALSE;

GLint repeatCount = 100;
GLint loopCount = 10;

GLubyte texture[4*3] = {
    0xFF, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0xFF, 0,
};


static void SetWindSize(int width, int height)
{

    windW = (GLint)width;
    windH = (GLint)height;
}

static GLenum Key(int key, GLenum mask)
{

    switch (key) {
      case TK_ESCAPE:
	tkQuit();
      case TK_a:
	antialiasing = !antialiasing;
#ifdef DEC /* bug fix */
	break;
#endif
      case TK_d:
	depthTesting = !depthTesting;
	break;
      case TK_f:
	fogging = !fogging;
#ifdef DEC /* bug fix */
	break;
#endif
      case TK_F:
	niceFogging = !niceFogging;
#ifdef DEC /* bug fix */
	break;
#endif
#ifdef DEC /* bug fix */
      case TK_l:
	lighting = !lighting;
	break;
#endif
      case TK_s:
	shading = !shading;
#ifdef DEC /* bug fix */
	break;
#endif
      case TK_t:
	texturing = !texturing;
#ifdef DEC /* bug fix */
	break;
#endif
      default:
	return GL_FALSE;
    }
    return GL_TRUE;
}

static void Viewport(GLint row, GLint column)
{
    GLint x, y;

    boxW = (windW - (COLS + 1) * GAP) / COLS;
    boxH = (windH - (ROWS + 1) * GAP) / ROWS;

    x = GAP + column * (boxW + GAP);
    y = GAP + row * (boxH + GAP);

    glViewport(x, y, boxW, boxH);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(-boxW/2, boxW/2, -boxH/2, boxH/2);
    glMatrixMode(GL_MODELVIEW);

    glEnable(GL_SCISSOR_TEST);
    glScissor(x, y, boxW, boxH);
}
#if defined (DEC) && defined(vms)
static double Now(void)
{
    static long	    start_time[2];
    long	    end_time[2];
    long	    diff[2];
    static int	    started = 0;

    if (started == 0)
    {
	started = 1;
	sys$gettim (start_time);
	return 0.0;
    }
    else
    {
	sys$gettim (end_time);
	started = 0;
	lib$sub_times (end_time, start_time, diff);
	/* diff[0] and diff[1] now have the difference in 100 ns units */
	if (diff[1] != ~0)
	{
	    fprintf (stderr, "Time too long\n");
	    exit (1);
	}
	return ((double)diff[0]) * -0.000000100; /* 100 ns == 0.000100 secs */
    }
}
#else	/* not vms */

static double Now(void)
{
#ifdef __unix
    struct tms tm;
    clock_t clk;

    clk = times(&tm);
    return (double)clk / (double)HZ;
#else
    return 0;
#endif
}
#endif /* not vms */

static void Report(const char *msg, float elapsed)
{

    if (elapsed == 0.0) {
	printf("%s per second: Unknown, elapsed time is zero\n", msg);
    } else {
	printf("%s per second: %g\n", msg, repeatCount*loopCount/elapsed);
    }
}

static void Points(void)
{
    GLint i, j;
    float v1[3];
    double start;

    start = Now();
    for (i = 0; i < repeatCount; i++) {
	v1[0] = 10;
	v1[1] = 10;
	v1[2] = 10;
	glBegin(GL_POINTS);
	    for (j = 0; j < loopCount; j++) {
		glVertex2fv(v1);
	    }
	glEnd();
    }
    glFinish();
    Report("Points", Now()-start);
}

static void Lines(void)
{
    GLint i, j;
    float v1[3], v2[3];
    double start;

    start = Now();
    for (i = 0; i < repeatCount; i++) {
	v1[0] = 10;
	v1[1] = 10;
	v1[2] = 10;
	v2[0] = 20;
	v2[1] = 20;
	v2[2] = 10;
	glBegin(GL_LINES);
	    for (j = 0; j < loopCount; j++) {
		glVertex2fv(v1);
		glVertex2fv(v2);
	    }
	glEnd();
    }
    glFinish();
    Report("Lines", Now()-start);
}

static void Triangles(void)
{
    GLint i, j;
    float v1[3], v2[3], v3[3], t1[2], t2[2], t3[2];
    double start;

    start = Now();

    v1[0] = 10;
    v1[1] = 10;
    v1[2] = 10;
    v2[0] = 20;
    v2[1] = 20;
    v2[2] = 10;
    v3[0] = 10;
    v3[1] = 20;
    v3[2] = 10;

    t1[0] = 0;
    t1[1] = 0;
    t2[0] = 1;
    t2[1] = 1;
    t3[0] = 0;
    t3[1] = 1;

    for (i = 0; i < repeatCount; i++) {
	glBegin(GL_TRIANGLES);
	    for (j = 0; j < loopCount; j++) {
		if (texturing) {
		    glTexCoord2fv(t1);
		}
		glVertex2fv(v1);
		if (texturing) {
		    glTexCoord2fv(t2);
		}
		glVertex2fv(v2);
		if (texturing) {
		    glTexCoord2fv(t3);
		}
		glVertex2fv(v3);
	    }
	glEnd();
    }
    glFinish();
    Report("Triangles", Now()-start);
}

static void Rects(void)
{
    GLint i, j;
    float v1[2], v2[2];
    double start;

    start = Now();
    for (i = 0; i < repeatCount; i++) {
	v1[0] = 10;
	v1[1] = 10;
	v2[0] = 20;
	v2[1] = 20;
	for (j = 0; j < loopCount; j++) {
	    glRectfv(v1, v2);
	}
    }
    glFinish();
    Report("Rects", Now()-start);
}

static void Draw(void)
{

    glClearColor(0.0, 0.0, 0.0, 0.0);
#ifdef DEC /* bug fix */
    glDisable(GL_SCISSOR_TEST);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    glEnable(GL_SCISSOR_TEST);
#else /* not DEC */
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
#endif

    TK_SETCOLOR(windType, TK_YELLOW);

    if (antialiasing) {
#ifdef DEC /* added print statements */
	printf ("\nAnti-aliased");
#endif
	glBlendFunc(GL_SRC_ALPHA, GL_ZERO);
	glEnable(GL_BLEND);

	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POLYGON_SMOOTH);
    }
#ifdef DEC /* bug fix */
    else
    {
	printf ("\nNot Anti-aliased");
	glDisable (GL_BLEND);
	glDisable (GL_POINT_SMOOTH);
	glDisable (GL_LINE_SMOOTH);
	glDisable (GL_POLYGON_SMOOTH);
    }
#endif /* DEC */
    if (depthTesting) {
#ifdef DEC /* added print statements */
	printf (", depth test");
#endif
	glEnable(GL_DEPTH_TEST);
    }
#ifdef DEC /* bug fix */
    else
    {
	printf (", no depth test");
	glDisable (GL_DEPTH_TEST);
    }
#endif /* DEC */
    if (fogging) {
#ifdef DEC /* added print statements */
	printf (", fog");
#endif
	glEnable(GL_FOG);
	glHint(GL_FOG_HINT, (niceFogging) ? GL_NICEST : GL_FASTEST);
    }
#ifdef DEC /* bug fix */
    else
    {
	printf (", no fog");
	glDisable (GL_FOG);
    }
#endif /* DEC */
    if (lighting) {
	static GLfloat ambient[4] = {1, 0.5, 0.5, 0};
#ifdef DEC /* added print statements */
	printf (", lighting");
#endif

	glEnable(GL_NORMALIZE);
	glNormal3f(1.0, 1.0, 1.0);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
    }
#ifdef DEC /* bug fix */
    else
    {
	printf (", no lighting");
	glDisable (GL_LIGHTING);
	glDisable (GL_LIGHT0);
    }
#endif /* DEC */
#ifdef DEC /* added print statements */
	if (shading) printf (", smooth shading");
	else printf (", flat shading");
#endif
    (shading) ? glShadeModel(GL_SMOOTH) : glShadeModel(GL_FLAT);
    if (texturing) {
	static GLfloat modulate[1] = {GL_DECAL};
	static GLfloat clamp[1] = {GL_CLAMP};
	static GLfloat linear[1] = {GL_LINEAR};
#ifdef DEC /* added print statements */
	printf (", texture\n");
#endif

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, 2, 2, 0, GL_RGB, GL_UNSIGNED_BYTE,
		     (GLvoid *)texture);
	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, modulate);
	glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, clamp);
	glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, clamp);
	glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, linear);
	glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, linear);
	glEnable(GL_TEXTURE_2D);
    }
#ifdef DEC /* bug fix */
    else
    {
	printf (", no texture\n");
	glDisable (GL_TEXTURE_2D);
    }
#endif /* DEC */

    Viewport(0, 0); Points();
    Viewport(0, 1); Lines();
    Viewport(0, 2); Triangles();
    Viewport(0, 3); Rects();

    glFlush();

    if (doubleBuffer) {
	tkSwapBuffers();
    }
}

static GLenum Args(int argc, char **argv)
{
    GLint i;

    rgb = GL_TRUE;
    doubleBuffer = GL_FALSE;
    directRender = GL_TRUE;

    for (i = 1; i < argc; i++) {
	if (strcmp(argv[i], "-ci") == 0) {
	    rgb = GL_FALSE;
	} else if (strcmp(argv[i], "-rgb") == 0) {
	    rgb = GL_TRUE;
	} else if (strcmp(argv[i], "-sb") == 0) {
	    doubleBuffer = GL_FALSE;
	} else if (strcmp(argv[i], "-db") == 0) {
	    doubleBuffer = GL_TRUE;
	} else if (strcmp(argv[i], "-dr") == 0) {
	    directRender = GL_TRUE;
	} else if (strcmp(argv[i], "-ir") == 0) {
	    directRender = GL_FALSE;
	} else {
	    printf("%s (Bad option).\n", argv[i]);
	    return GL_FALSE;
	}
    }
    return GL_TRUE;
}

#ifndef DEC
void main(int argc, char **argv)
#else /* DEC */
int main(int argc, char **argv)
#endif /* DEC */
{

    if (Args(argc, argv) == GL_FALSE) {
	tkQuit();
    }

    windW = 600;
    windH = 300;
    tkInitPosition(0, 0, windW, windH);

    windType = TK_ALPHA | TK_DEPTH;
    windType |= (rgb) ? TK_RGB : TK_INDEX;
    windType |= (doubleBuffer) ? TK_DOUBLE : TK_SINGLE;
    windType |= (directRender) ? TK_DIRECT : TK_INDIRECT;
    tkInitDisplayMode(windType);

    if (tkInitWindow("Speed Test") == GL_FALSE) {
	tkQuit();
    }

    tkExposeFunc(SetWindSize);
    tkReshapeFunc(SetWindSize);
    tkKeyDownFunc(Key);
    tkDisplayFunc(Draw);
    tkExec();
}
