/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: double.c,v $
 * Revision 1.1.2.6  1994/01/21  14:57:24  John_Dennis
 * 	brought file up to SGI 1.0.12 release
 * 	[1994/01/21  00:33:17  John_Dennis]
 *
 * Revision 1.1.2.5  1994/01/14  16:46:27  Andy_Vesper
 * 	Don't use <X11/GLw/...> on VMS
 * 	[1994/01/13  21:55:24  Andy_Vesper]
 * 
 * Revision 1.1.2.4  1993/10/26  21:15:09  John_Dennis
 * 	change include for GLw includes to <X11/GLw/*.h>
 * 	[1993/10/26  21:09:22  John_Dennis]
 * 
 * Revision 1.1.2.3  1993/10/06  21:18:40  John_Dennis
 * 	brought source up to SGI release 1.0.11
 * 	[1993/10/06  21:05:49  John_Dennis]
 * 
 * Revision 1.1.2.2  1993/05/20  16:40:02  John_Dennis
 * 	new files in SGI OpenGL 1.0.9 release
 * 	[1993/05/20  15:10:09  John_Dennis]
 * 
 * $EndLog$
 */
#include <stdio.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <X11/keysym.h>
#if defined(DEC) && defined (__vms)
#include "GLwMDrawA.h"
#else
#include <X11/GLw/GLwMDrawA.h>
#endif

/* Callbacks */
static void draw_scene_callback();
static void do_resize();
static void init_window();
static void input();


static XtAppContext app_context;
static String fallback_resources[] = {
    "*frame*shadowType: SHADOW_IN",
    "*GLwMDrawingArea*width: 100",
    "*GLwMDrawingArea*height: 100",
    "*GLwMDrawingArea*rgba: TRUE",
    "*GLwMDrawingArea*allocateBackground: TRUE",
    "*glwidget*background:red",
    "*glwidget2*background:green",
    NULL
    };

main(argc, argv)
int argc;
char *argv[];
{
    Arg args[20];
    int n;
    Widget glw, toplevel, form, frame;


    toplevel = XtAppInitialize(&app_context, "4DgiftsGLw", 
			       (XrmOptionDescList)NULL , 0,
			       &argc, 
			       (String*)argv, 
			       fallback_resources,
			       (ArgList)NULL, 0);

    n = 0;
    form = XmCreateForm(toplevel, "form", args, n);
    XtManageChild(form);

    n = 0;
    XtSetArg(args[n], XtNx, 30); n++;
    XtSetArg(args[n], XtNy, 30); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, 30); n++;
    XtSetArg(args[n], XmNtopOffset, 30); n++;
    XtSetArg(args[n], XmNrightOffset, 30); n++;
    frame = XmCreateFrame (form, "frame", args, n);
    XtManageChild (frame);

    n = 0;
    glw = GLwCreateMDrawingArea(frame, "glwidget", args, n);
    XtManageChild (glw);
    XtAddCallback(glw, GLwNexposeCallback, draw_scene_callback, 0);
    XtAddCallback(glw, GLwNresizeCallback, do_resize, 0);
    XtAddCallback(glw, GLwNginitCallback, init_window, 0);
    XtAddCallback(glw, GLwNinputCallback, input, 0);

    n = 0;
    XtSetArg(args[n], XtNx, 30); n++;
    XtSetArg(args[n], XtNy, 30); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNtopWidget, frame); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, 30); n++;
    XtSetArg(args[n], XmNtopOffset, 30); n++;
    XtSetArg(args[n], XmNbottomOffset, 30); n++;
    XtSetArg(args[n], XmNrightOffset, 30); n++;
    frame = XmCreateFrame (form, "frame", args, n);
    XtManageChild (frame);

    n = 0;
    glw = GLwCreateMDrawingArea(frame, "glwidget2", args, n);
    XtManageChild (glw);
    XtAddCallback(glw, GLwNexposeCallback, draw_scene_callback, 0);
    XtAddCallback(glw, GLwNresizeCallback, do_resize, 0);
    XtAddCallback(glw, GLwNginitCallback, init_window, 0);
    XtAddCallback(glw, GLwNinputCallback, input, 0);

    XtRealizeWidget(toplevel);

    XtAppMainLoop(app_context);

}


static void
init_window(Widget w, XtPointer client_data, XtPointer call_data)
{
    Arg args[1];
    XVisualInfo *vi;
    GLXContext glx_context;

    printf ("ginit\n");

    XtSetArg(args[0], GLwNvisualInfo, &vi);
    XtGetValues(w, args, 1);

    glx_context = glXCreateContext(XtDisplay(w), vi, 0, GL_FALSE);
    XtSetArg(args[0], XmNuserData, glx_context);
    XtSetValues(w, args, 1);
}


static void
draw_scene_callback(Widget w, XtPointer client_data, XtPointer call_data)
{
    GLXContext glx_context;
    Arg args[1];

    XtSetArg(args[0], XmNuserData, &glx_context);
    XtGetValues(w, args, 1);

    GLwDrawingAreaMakeCurrent(w, glx_context);

#ifdef notdef
    /* clear the buffer	*/
    glClearColor(1,1,0,1);
    glClear(GL_COLOR_BUFFER_BIT);
    glFlush();
#endif
}

static void
do_resize(Widget w, XtPointer client_data,
	  GLwDrawingAreaCallbackStruct *call_data)
{
    printf ("resize to %d, %d\n", call_data->width, call_data->height);
}

/* Process all Input callbacks*/
static void
input(Widget w, XtPointer client_data, GLwDrawingAreaCallbackStruct *call_data)
{
    char buffer[1];
    KeySym keysym;

    switch(call_data->event->type)
    {
    case KeyRelease:
	/* It is necessary to convert the keycode to a keysym before
	 * it is possible to check if it is an escape
	 */
	if (XLookupString((XKeyEvent*)call_data->event,buffer,1,&keysym,NULL)
	    == 1 && keysym == (KeySym)XK_Escape) {
	  exit(0);
      }
	break;
    }
}


